import { Plugin } from 'ckeditor5/src/core';
import { createDropdown, addListToDropdown, ContextualBalloon, clickOutsideHandler } from 'ckeditor5/src/ui';
import { Collection } from 'ckeditor5/src/utils';
import templateIcon from '../../../../icons/blocks.svg';
import FormView from './insert_blocks-view';

export default class InsertBlocks extends Plugin {
  async init() {
    const editor = this.editor;

    // Allowed class and data-* on div
    editor.config.set('htmlSupport', {
      allow: [
        {
          name: 'div',
          attributes: {
            class: true,
            'data-block-id': true,
            'data-library': true,
          },
          classes: true,
        },
      ],
    });

    // Get blocks name from editor config.
    const blocks = this.editor.config.get('blocks');

    editor.ui.componentFactory.add('insertBlocks', (locale) => {
      const dropdown = createDropdown(locale);

      dropdown.buttonView.set({
        label: editor.t('Insert block'),
        withText: false,
        icon: templateIcon,
        tooltip: true
      });

      // Add block name to dropdown
      addListToDropdown(dropdown, this._createBlockItems(blocks));

      dropdown.on('execute', async (event) => {
        await this._onExecute(event);
      });

      return dropdown;
    });
    // Create balloon panel to add class
    this._balloon = editor.plugins.get(ContextualBalloon);
  }

  _createBlockItems(blocks) {
    const collection = new Collection();

    Object.entries(blocks).forEach(([blockId, label]) => {
      const templateElement = {
        label: label,
        withText: true,
        icon: templateIcon,
        tooltip: label,
        blockId: blockId
      };

      collection.add({
        type: 'button',
        model: templateElement
      });
    });

    return collection;
  }

  async _onExecute(event) {
    const editor = this.editor;
    const blockId = event.source.blockId;
    const urlGetContent = editor.config.get('url_block_content') + '/' + blockId;

    let loadingElementPosition;

    // Insert "Loading..." to editor
    editor.model.change((writer) => {
      const loadingContent = `<div class="loading insert-block" data-block-id="${blockId}">Loading... ${editor.config.get('blocks')[blockId]}</div>`;
      const loadingFragment = editor.data.processor.toView(loadingContent);
      const loadingModelFragment = editor.data.toModel(loadingFragment);
      loadingElementPosition = editor.model.document.selection.getFirstPosition();
      writer.insert(loadingModelFragment, loadingElementPosition);
    });

    try {
      // Load content block from server
      const response = await fetch(urlGetContent);
      const blockContent = await response.text();

      // Wrapper block with class and data block id.
      const wrappedContent = `<div class="insert-block" data-block-id="${blockId}">${blockContent}</div>`;

      // Replace "Loading..." by block content
      editor.model.change((writer) => {
        const rangeToRemove = writer.createRange(
          loadingElementPosition,
          loadingElementPosition.getShiftedBy(1)
        );
        writer.remove(rangeToRemove);
        const viewFragment = editor.data.processor.toView(wrappedContent);
        const modelFragment = editor.data.toModel(viewFragment);
        writer.insert(modelFragment, loadingElementPosition);
        this.blockId = blockId;
        this.formView = this._createBalloonContent(loadingElementPosition);
        // Show balloon to add class after content is inserted
        this._showBalloon(loadingElementPosition);
      });

    } catch (error) {
      console.error('Error loading block content:', error);
    }
  }

  _createBalloonContent(position) {
    // Create balloon with input class
    const editor = this.editor;
    const formView = new FormView(editor);

    // On submit send the user data to the writer, then hide the form view.
    this.listenTo(formView, 'submit', () => {
      let form = {
        'class': formView.classInputView.fieldView.element.value,
        'library': formView.libraryInputView.fieldView.element.value,
        'block_id': this.blockId,
        'position': position
      };
      editor.execute('InsertBlockCommand', form);
      this._hideUI();
    });

    // Hide the form view after clicking the "Cancel" button.
    this.listenTo(formView, 'cancel', () => {
      this._hideUI();
    });

    // Hide the form view when clicking outside the balloon.
    clickOutsideHandler({
      emitter: formView,
      activator: () => this._balloon.visibleView === formView,
      contextElements: [this._balloon.view.element],
      callback: () => this._hideUI()
    });
    return formView;
  }

  _showBalloon(position) {
    const editor = this.editor;
    const targetElement = editor.editing.view.document.selection.getFirstRange();

    // Add balloon to contextual balloon
    if (targetElement) {
      this._balloon.add({
        view: this.formView,
        position: this._getBalloonPositionData(position),
      });
      this.formView.focus();
    }
  }

  _hideUI() {
    this.formView.classInputView.fieldView.value = '';
    this.formView.libraryInputView.fieldView.value = '';
    this.formView.element.reset();
    this._balloon.remove(this.formView);

    // Focus the editing view after closing the form view.
    this.editor.editing.view.focus();
  }

  _getBalloonPositionData(position) {
    const view = this.editor.editing.view;
    const viewDocument = view.document;
    let target = null;

    // Set a target position by converting view selection range to DOM.
    target = () => view.domConverter.viewRangeToDom(
      viewDocument.selection.getFirstRange()
    );

    return {
      target
    };
  }

  static get requires() {
    return [ContextualBalloon];
  }

}
