<?php

namespace Drupal\ckeditor_insert_blocks\Plugin\Filter;

use Drupal\Core\Block\BlockManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DomCrawler\Crawler;

/**
 * Insert block filter.
 */
#[Filter(
  id: "insert_block",
  title: new TranslatableMarkup("Insert block"),
  type: FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
  description: new TranslatableMarkup("It uses for ckeditor 5 filter."),
)]
class InsertBlockFilter extends FilterBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new InsertBlockFilter instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected BlockManagerInterface $pluginManagerBlock,
    protected RendererInterface $renderer,
    protected AccountProxyInterface $currentUser,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.block'),
      $container->get('renderer'),
      $container->get('current_user'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $response = new FilterProcessResult();
    if (!class_exists('Crawler')) {
      $render = [
        '#type' => 'inline_template',
        '#template' => $text,
      ];
      $text = $this->renderer->renderInIsolation($render);
      return $response->setProcessedText($text);
    }
    // Initialize the Crawler with the provided text.
    $crawler = new Crawler($text);
    $lib = [];

    // Use XPath to find div elements with class "insert-block & data-block-id".
    $xPath = '//div[contains(@class, "insert-block") and @data-block-id]';
    $crawler->filterXpath($xPath)->each(function (Crawler $node) use (&$lib) {
      // Get the block ID.
      $blockId = $node->attr('data-block-id');

      if (!empty($blockId)) {
        // Render the block content.
        $blockContent = $this->renderBlock($this->currentUser, $blockId);

        // Get the DOM node and check if it exists.
        $domNode = $node->getNode(0);
        if ($domNode) {
          // Replace the content safely.
          while ($domNode->hasChildNodes()) {
            $domNode->removeChild($domNode->firstChild);
          }

          // Create a new DOM fragment.
          $fragment = $domNode->ownerDocument->createDocumentFragment();
          $fragment->appendXML($blockContent);
          $domNode->appendChild($fragment);
        }
      }

      // Process libraries if data-library attribute exists.
      $libraries = $node->attr('data-library');
      if (!empty($libraries)) {
        foreach (explode(',', $libraries) as $library) {
          $lib[] = trim($library);
        }
      }
    });

    // Prepare the render array with the updated HTML.
    $text = $crawler->html();
    // Attach libraries if any.
    if (!empty($lib)) {
      $render = [
        '#type' => 'inline_template',
        '#template' => $text,
      ];
      $render['#attached']['library'] = array_unique($lib);
      // Render the updated content in isolation.
      $text = $this->renderer->renderInIsolation($render);
    }
    // Return the processed text.
    return $response->setProcessedText($text);
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    if ($long) {
      return $this->t('Attribute data-block-id is required to display the block content. You need to clear the cache to see the effect');
    }

    return $this->t('Displays and renders block HTML markup.');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('Uses a data-block-id attribute on &lt;div&gt; tags to displays and renders block HTML markup');
  }

  /**
   * Render block with current user and block id.
   *
   * @inheritDoc
   */
  public function renderBlock($currentUser, $delta) {
    $plugin_block = $this->pluginManagerBlock->createInstance($delta, []);
    $render = '';
    if ($plugin_block->access($currentUser)) {
      $render = $plugin_block->build();
      if (!empty($render["#view"]) && empty($render["#view"]->result)) {
        return FALSE;
      }
      if (empty($render)) {
        return FALSE;
      }
      if (is_array($render)) {
        return $this->renderer->renderInIsolation($render);
      }
    }

    return $render;
  }

}
